/*
  tests_ggr.c
  Copyright (c) J.J. Green 2014
*/

#include <stdio.h>
#include <unistd.h>

#include <ggr.h>

#include "tests_ggr.h"

#include "fixture.h"
#include "fixture_ggr.h"

CU_TestInfo tests_ggr[] =
  {
    {"load", test_ggr_load},
    {"save", test_ggr_save},
    {"sample at midpoint", test_ggr_sample_at_midpoint},
    {"sample outside range", test_ggr_sample_outside_range},
    {"sample null", test_ggr_sample_null},
    CU_TEST_INFO_NULL
  };

extern void test_ggr_load(void)
{
  size_t i, n = 1024;
  char path[n];

  const char *ggrs[] = {"Sunrise.ggr", "mars.ggr"};

  for (i = 0 ; i < 2  ; i++)
    {
      CU_TEST_FATAL( fixture(path, n, "ggr", ggrs[i]) < n );

      gradient_t *grad;
      CU_TEST_FATAL( (grad = grad_load_gradient(path)) != NULL );
      grad_free_gradient(grad);
    }
}

extern void test_ggr_save(void)
{
  gradient_t *ggr;
  CU_TEST_FATAL( (ggr = load_ggr_fixture("Sunrise.ggr")) != NULL );

  char *path = tmpnam(NULL);
  CU_ASSERT( grad_save_gradient(ggr, path) == 0 );

  unlink(path);
  grad_free_gradient(ggr);
}

extern void test_ggr_sample_at_midpoint(void)
{
  gradient_t *ggr;
  CU_TEST_FATAL( (ggr = load_ggr_fixture("Sunrise.ggr")) != NULL );

  double bg[3] = {0, 0, 0}, rgbD[3], eps = 1e-6;

  CU_ASSERT( gradient_colour(0.5, ggr, bg, rgbD) == 0 );

  CU_ASSERT_DOUBLE_EQUAL(rgbD[0], 0.837332, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[1], 0.375108, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[2], 0.231127, eps);

  grad_free_gradient(ggr);
}

extern void test_ggr_sample_outside_range(void)
{
  gradient_t *ggr;
  CU_TEST_FATAL( (ggr = load_ggr_fixture("Sunrise.ggr")) != NULL );

  double bg[3] = {0, 0, 0}, rgbD[3], eps = 1e-8;

  CU_ASSERT( gradient_colour(-0.5, ggr, bg, rgbD) == 0 );

  CU_ASSERT_DOUBLE_EQUAL(rgbD[0], 1, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[1], 1, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[2], 1, eps);

  grad_free_gradient(ggr);
}

extern void test_ggr_sample_null(void)
{
  double bg[3] = {0, 0.5, 1.0}, rgbD[3], eps = 1e-8;

  CU_ASSERT( gradient_colour(0.5, NULL, bg, rgbD) == 0 );

  CU_ASSERT_DOUBLE_EQUAL(rgbD[0], 0.0, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[1], 0.5, eps);
  CU_ASSERT_DOUBLE_EQUAL(rgbD[2], 1.0, eps);
}
